<?php
/**	
 *	Form Creator Class
 *
 *	Helper class for form creation & validation
 *
 */
class form_creator {


	var $form 		= '';

	var $errors 	= array(),
		$elements 	= array(),
		$default_element_attr = array(
			'name'		=> null,		// element name
			'id'		=> null,		// element id
			'class'		=> null,		// element class
			'errclass'	=> 'invalid', 	// class added to invalid form elements
			'label'		=> null,		// element label text						
			'default'	=> null,		// default value or selected item in select dropbox
			'required'	=> false,		// is element required field
			'validation'=> array(		// validation rules and error msg goes here
					'filter' 	=> false,
					'regex' 	=> false,
					'msg' 		=> false
				),
			'items'	=> array(), 		// stores val => desc for select and radio
			'value' => null,			// save post/get value to this var
			'valid' => false 			// save validation result here
		);

	/**
	 *	Constructor
	 *
	 *	@param 	string 	formname -not used anywhere, ever.
	 */
	function __construct( $formname ){

		$this->form = $formname;

	}

	/**
	 *	Add Form Element
	 *
	 *	@param 	string 	uniq id for element
	 *	@param 	array 	element options
	 */
	function add_element( $id, $attr ) {
		$attr['name']	= $id;
		$attr['id']		= $id;

		$this->elements[$id] = array_merge($this->default_element_attr, $attr);
	}

	/**
	 *	Get Form Element
	 *
	 *	Calls element type render function
	 *
	 *	@param 	string 	uniq id for element
	 */
	function get_element($id){
		// check if element exists
		if(!is_array($this->elements[$id])){
			trigger_error("Element '" . $id . "' does not exist", E_USER_NOTICE);
			return false;
		} elseif(!method_exists($this, 'render_'.$this->elements[$id]['type'])) {
			// element does not exist or render method does not exist
			trigger_error("Render method does not exist: " . 'render_'.$this->elements[$id]['type'], E_USER_ERROR);
			return false;
		} else {
			// call element type render function
			echo $this->{'render_'.$this->elements[$id]['type']}($this->elements[$id]);
			return true;
		}
	}

	/**
	 *	Get Values
	 *
	 *	Saves $_POST values to elements array
	 *
	 * 	@return bool true if a value was saved
	 */
	function get_values(){
		$post = false;
		foreach ($this->elements as $id => $e) {
			// save each element value from post
			if(isset($_POST[$e['name']])) { $this->elements[$id]['value'] = $_POST[$e['name']]; $post = true; }
		}
		return $post;
	}

	/**
	 *	Get Label
	 *
	 *	Render label for element with id
	 *
	 *	@param 	string 	uniq id for element
	 */
	function get_label($id){
		if(is_array($this->elements[$id])){
			echo '<label for="'.$this->elements[$id]['id'].'">'.$this->elements[$id]['label'].'</label>';
		} else {
			return false;
		}
	}

	/**
	 *	Get Errors
	 *
	 *	Returns formatted list of validation errors
	 *
	 *	@param 	string 	before list
	 *	@param 	string 	after list
	 *	@param 	string 	before list item
	 *	@param 	string 	after list item
	 * 	@return string
	 */
	function get_errors($pre = "<ul>", $post = "</ul>", $item_pre = "<li>", $item_post = "</li>"){
		if($this->has_errors()) { 
			return $pre.$item_pre.implode($item_post . $item_pre, $this->errors).$item_post.$post;
		} else {
			return '';
		}
		
	}

	/**
	 *	Get Results
	 *
	 *	Returns assoc array with custom key
	 *
	 *	@param 	string 	element attr as key (id, name, label etc.)
	 * 	@return array
	 */
	function get_results($id = false){
		$id = $id? $id :'name';
		if(!array_key_exists($id,$this->default_element_attr)) { $id = 'name'; }

		$results = array();
		foreach($this->elements as $el){
			$results[$el[$id]] = $el['value'];
		}
		return $results;
	}

	/**
	 *	Validate Form
	 *
	 *	Check all elements and save any validation errors
	 *
	 *	@return bool
	 */
	function validate(){
		foreach($this->elements as $id => $el){
			if($el['required']) {
				if(isset($el['validation']['filter'])) {
					$this->validate_element_filter($id);
				} elseif(isset($el['validation']['regex'])) {
					$this->validate_element_regex($id);
				}
			}
		}
		return !$this->has_errors();
	}

	/**
	 *	Form Has Errors
	 *
	 *	Returns true if there were errors in form validation
	 *
	 *	@return bool
	 */
	function has_errors(){
		return !empty($this->errors);
	}

	function validate_element_filter($id) {
		$type 	= $this->elements[$id]['validation']['filter'];
		$value 	= $this->elements[$id]['value'];
		$msg 	= $this->elements[$id]['validation']['msg'];

		$filters = array(
				"email" => FILTER_VALIDATE_EMAIL,
				"bool"	=> FILTER_VALIDATE_BOOLEAN,
				"int" 	=> FILTER_VALIDATE_INT,
				"url"	=> FILTER_VALIDATE_URL
		);

		// Shorthand use predefined filters below or litteral filter
		$filter = array_key_exists($type,$filters) ? $filters[$type] : $type;

		if(!filter_var($value, $filter)) {  
			$this->errors[$id] = $msg;
			$this->elements[$id]['valid'] = false;
		}
	}

	function validate_element_regex($id) {
		$reg 	= $this->elements[$id]['validation']['regex'];
		$value 	= $this->elements[$id]['value'];
		$msg 	= $this->elements[$id]['validation']['msg'];

		if(!preg_match($reg, $value)) {  
			$this->errors[$id] = $msg;
			$this->elements[$id]['valid'] = false;
		}
	}

	function render_input($attr){
		return '<input type="text" name="'.$attr['name'].'" id="'.$attr['id'].'" class="'.$attr['class'].(array_key_exists($attr['id'], $this->errors) ? ' '.$attr['errclass']:'').'" value="'.$attr['value'].'">';
	}

	function render_select($attr){
		$items = "";
		// check if a value has been set otherwise set default as selected
		$selected = ($attr['value'] != '' ? $attr['value'] : $attr['default']);

		if(is_array($attr['items'])) {
			foreach ( $attr['items'] as $val => $desc ) {
				$items .= '<option value="'.$val.'"'. ( $selected == $val ? ' selected="selected"':'' ) .'>'.$desc.'</option>';
			}
		}
		return '<select name="'.$attr['name'].'" id="'.$attr['id'].'" class="'.$attr['class'].'">'.$items.'</select>';
	}

	function render_checkbox($attr){
		return '<input type="checkbox" name="'.$attr['name'].'" id="'.$attr['id'].'" class="'.$attr['class'].'" value="'.$attr['default'].'"'.( $attr['value'] == $attr['default'] ? 'checked="checked"':'' ).'>';
	}

	function render_textarea($attr){
		return '<textarea name="'.$attr['name'].'" id="'.$attr['id'].'" class="'.$attr['class'].(array_key_exists($attr['id'], $this->errors) ? ' '.$attr['errclass']:'').'">'.$attr['value'].'</textarea>';
	}
}

